/**
 * Open SUIT - Simple User Interface Toolkit
 * 
 * Copyright (c) 2009 France Telecom, http://www.francetelecom.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package org.ow2.opensuit.xml.base.locale;

import java.nio.charset.Charset;
import java.nio.charset.IllegalCharsetNameException;
import java.nio.charset.UnsupportedCharsetException;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Locale;

import org.ow2.opensuit.core.session.ILocaleConfig;
import org.ow2.opensuit.xmlmap.annotations.XmlAttribute;
import org.ow2.opensuit.xmlmap.annotations.XmlDoc;
import org.ow2.opensuit.xmlmap.annotations.XmlElement;
import org.ow2.opensuit.xmlmap.interfaces.IInitializable;
import org.ow2.opensuit.xmlmap.interfaces.IInitializationSupport;



@XmlDoc("Component that defines locale specific settings.")
@XmlElement
public class LocaleConfig implements ILocaleConfig, IInitializable
{
	@XmlDoc("Language code (ex: 'en', 'fr', ...)")
	@XmlAttribute(name="Language")
	private String language;
	
	@XmlDoc("Country code (ex: 'us', 'uk', ...)")
	@XmlAttribute(name="Country", required=false)
	private String country;
	
	@XmlDoc("Character encoding to use with this language.")
	@XmlAttribute(name="CharSet", required=false)
	private String charSet= "iso-8859-1";
	
//	private String _xmlattr_opt_TimeZone;
	@XmlDoc("The date output format (see java.text.SimpleDateFormat).")
	@XmlAttribute(name="DateOutputFormat", required=false)
	private String dateOutputFormat;
	
	@XmlDoc("The date input format (see java.text.SimpleDateFormat).")
	@XmlAttribute(name="DateInputFormat", required=false)
	private String dateInputFormat;
	
	@XmlDoc("The time output format (see java.text.SimpleDateFormat).")
	@XmlAttribute(name="TimeOutputFormat", required=false)
	private String timeOutputFormat;
	
	@XmlDoc("The time input format (see java.text.SimpleDateFormat).")
	@XmlAttribute(name="TimeInputFormat", required=false)
	private String timeInputFormat;
	
	@XmlDoc("The date and time output format (see java.text.SimpleDateFormat).")
	@XmlAttribute(name="DateTimeOutputFormat", required=false)
	private String dateTimeOutputFormat;
	
	@XmlDoc("The date and time input format (see java.text.SimpleDateFormat).")
	@XmlAttribute(name="DateTimeInputFormat", required=false)
	private String dateTimeInputFormat;
	
	private java.util.Locale locale;
//	private TimeZone _timeZone;
	private DateFormat _dateOutputFormat;
	private DateFormat _dateInputFormat;
	private DateFormat _timeOutputFormat;
	private DateFormat _timeInputFormat;
	private DateFormat _dateTimeOutputFormat;
	private DateFormat _dateTimeInputFormat;
	// TODO: currency ?
	
	public void initialize(IInitializationSupport initSupport, Object additionalSupport)
	{
		locale = country == null ? new Locale(language) : new Locale(language, country);
		
		if(dateOutputFormat != null)
			try {
				_dateOutputFormat = new SimpleDateFormat(dateOutputFormat);
			} catch(IllegalArgumentException e) {
				initSupport.addValidationMessage(this, "DateOutputFormat", IInitializationSupport.ERROR, "Invalid Date Output Format: '"+dateOutputFormat+"'");
			}
		else
			_dateOutputFormat = DateFormat.getDateInstance(DateFormat.MEDIUM, locale);
		
		if(dateInputFormat != null)
			try {
				_dateInputFormat = new SimpleDateFormat(dateInputFormat);
			} catch(IllegalArgumentException e) {
				initSupport.addValidationMessage(this, "DateInputFormat", IInitializationSupport.ERROR, "Invalid Date Input Format: '"+dateInputFormat+"'");
			}
		else
			_dateInputFormat = _dateOutputFormat;
		
		if(timeOutputFormat != null)
			try {
				_timeOutputFormat = new SimpleDateFormat(timeOutputFormat);
			} catch(IllegalArgumentException e) {
				initSupport.addValidationMessage(this, "TimeOutputFormat", IInitializationSupport.ERROR, "Invalid Time Output Format: '"+timeOutputFormat+"'");
			}
		else
			_timeOutputFormat = DateFormat.getTimeInstance(DateFormat.MEDIUM, locale);
		
		if(timeInputFormat != null)
			try {
				_timeInputFormat = new SimpleDateFormat(timeInputFormat);
			} catch(IllegalArgumentException e) {
				initSupport.addValidationMessage(this, "TimeInputFormat", IInitializationSupport.ERROR, "Invalid Time Input Format: '"+timeInputFormat+"'");
			}
		else
			_timeInputFormat = _timeOutputFormat;
		
		if(dateTimeOutputFormat != null)
			try {
				_dateTimeOutputFormat = new SimpleDateFormat(dateTimeOutputFormat);
			} catch(IllegalArgumentException e) {
				initSupport.addValidationMessage(this, "DateTimeOutputFormat", IInitializationSupport.ERROR, "Invalid Date Time Output Format: '"+dateTimeOutputFormat+"'");
			}
		else
			_dateTimeOutputFormat = DateFormat.getDateTimeInstance(DateFormat.MEDIUM, DateFormat.MEDIUM, locale);

		if(dateTimeInputFormat != null)
			try {
				_dateTimeInputFormat = new SimpleDateFormat(dateTimeInputFormat);
			} catch(IllegalArgumentException e) {
				initSupport.addValidationMessage(this, "DateTimeInputFormat", IInitializationSupport.ERROR, "Invalid Date Time Input Format: '"+dateTimeInputFormat+"'");
			}
		else
			_dateTimeInputFormat = _dateTimeOutputFormat;
		
		// --- check charset
		try
		{
			Charset.forName(charSet);
		}
		catch(IllegalCharsetNameException e)
		{
			initSupport.addValidationMessage(this, "CharSet", IInitializationSupport.ERROR, "Invalid CharSet: '"+charSet+"'");
		}
		catch(UnsupportedCharsetException e)
		{
			initSupport.addValidationMessage(this, "CharSet", IInitializationSupport.ERROR, "Unsupported CharSet: '"+charSet+"'");
		}
		
		/*
		if(_xmlattr_opt_TimeZone != null)
		{
			timeZone = TimeZone.getTimeZone(_xmlattr_opt_TimeZone);
			
			dateOutputFormat.setTimeZone(timeZone);
			dateInputFormat.setTimeZone(timeZone);
			timeOutputFormat.setTimeZone(timeZone);
			timeInputFormat.setTimeZone(timeZone);
			dateTimeOutputFormat.setTimeZone(timeZone);
			dateTimeInputFormat.setTimeZone(timeZone);
		}
		*/
	}
	public java.util.Locale getLocale()
    {
    	return locale;
    }

	public DateFormat getDateInputFormat()
    {
    	return _dateInputFormat;
    }

	public DateFormat getDateOutputFormat()
    {
    	return _dateOutputFormat;
    }

	public DateFormat getDateTimeInputFormat()
    {
    	return _dateTimeInputFormat;
    }

	public DateFormat getDateTimeOutputFormat()
    {
    	return _dateTimeOutputFormat;
    }

	public DateFormat getTimeInputFormat()
    {
    	return _timeInputFormat;
    }

	public DateFormat getTimeOutputFormat()
    {
    	return _timeOutputFormat;
    }
	public String getCharSet()
	{
	    return charSet;
	}
}
